///////////////////////////////////////////////////////////////////////////
// ASVR: Active Set Support Vector Regression                            //
// Copyright 2002 David R. Musicant and Alexander Feinberg.              //
// This software is free for academic use only.                          //
// For commercial use, contact musicant@cs.wisc.edu.                     //
///////////////////////////////////////////////////////////////////////////
// StandAlone.H: Header for code dealing with standalone aspects of the  //
// program (i.e., not the mexfile).                                      //
///////////////////////////////////////////////////////////////////////////

#ifndef STANDALONE_H
#define STANDALONE_H

void ThrowIt(const char *text);

#endif
