///////////////////////////////////////////////////////////////////////////
// ASVR: Active Set Support Vector Regression                            //
// Copyright 2002 David R. Musicant and Alexander Feinberg.              //
// This software is free for academic use only.                          //
// For commercial use, contact dmusican@carleton.edu.                    //
///////////////////////////////////////////////////////////////////////////
// StandAlone.cpp: Code dealing with standalone aspects of the program   //
// (i.e., not the mexfile).                                              //
///////////////////////////////////////////////////////////////////////////

// Wrapper function for throwing an exception. Since exceptions are not
// properly supported in mex files, they are handled differently there.
void ThrowIt(const char *text) {
  throw text;
}
