####################################################################
#  LAPACK make include file.                                       #
#  LAPACK, Version 2.0                                             #
#  September 30, 1994                                              #
####################################################################
#  Modifed by David R. Musicant to contain those parts necessary   #
#  for the ASVM project.                                           #
####################################################################

#  
#  Modify the CC and CFLAGS definitions to refer to the
#  compiler and desired compiler options for your machine.  NOOPT
#  refers to the compiler options desired when NO OPTIMIZATION is
#  selected.  Define LOADER and LOADOPTS to refer to the loader and 
#  desired load options for your machine.
#
#######################################################
# This is used to compile C libary
CC        = cc
CFLAGS    = -O2
LOADER    = cc
LOADOPTS  =
#######################################################################
#
#  The archiver and the flag(s) to use when building archive (library)
#  If you system has no ranlib, set RANLIB = echo.
#
ARCH     = ar
ARCHFLAGS= cr
RANLIB   = ranlib
#
#  The location of the libraries to which you will link.  (The 
#  machine-specific, optimized BLAS library should be used whenever
#  possible.)
#
BLASLIB      = ../libblas.a
LAPACKLIB    = ../liblapack.a
